"""


Application Handlers (bpy.app.handlers)
***************************************

This module contains callback lists


Basic Handler Example
=====================

This script shows the most simple example of adding a handler.

.. code::


  import bpy


  def my_handler(scene):
      print("Frame Change", scene.frame_current)


  bpy.app.handlers.frame_change_pre.append(my_handler)


Persistent Handler Example
==========================

By default handlers are freed when loading new files, in some cases you may
want the handler stay running across multiple files (when the handler is
part of an add-on for example).

For this the :data:`bpy.app.handlers.persistent` decorator needs to be used.

.. code::


  import bpy
  from bpy.app.handlers import persistent


  @persistent
  def load_handler(dummy):
      print("Load Handler:", bpy.data.filepath)


  bpy.app.handlers.load_post.append(load_handler)

:data:`depsgraph_update_post`

:data:`depsgraph_update_pre`

:data:`frame_change_post`

:data:`frame_change_pre`

:data:`load_factory_preferences_post`

:data:`load_factory_startup_post`

:data:`load_post`

:data:`load_pre`

:data:`redo_post`

:data:`redo_pre`

:data:`render_cancel`

:data:`render_complete`

:data:`render_init`

:data:`render_post`

:data:`render_pre`

:data:`render_stats`

:data:`render_write`

:data:`save_post`

:data:`save_pre`

:data:`undo_post`

:data:`undo_pre`

:data:`version_update`

:data:`persistent`

"""

import typing

depsgraph_update_post: typing.Any = ...

"""

on depsgraph update (post)

"""

depsgraph_update_pre: typing.Any = ...

"""

on depsgraph update (pre)

"""

frame_change_post: typing.Any = ...

"""

on frame change for playback and rendering (after)

"""

frame_change_pre: typing.Any = ...

"""

on frame change for playback and rendering (before)

"""

load_factory_preferences_post: typing.Any = ...

"""

on loading factory preferences (after)

"""

load_factory_startup_post: typing.Any = ...

"""

on loading factory startup (after)

"""

load_post: typing.Any = ...

"""

on loading a new blend file (after)

"""

load_pre: typing.Any = ...

"""

on loading a new blend file (before)

"""

redo_post: typing.Any = ...

"""

on loading a redo step (after)

"""

redo_pre: typing.Any = ...

"""

on loading a redo step (before)

"""

render_cancel: typing.Any = ...

"""

on canceling a render job

"""

render_complete: typing.Any = ...

"""

on completion of render job

"""

render_init: typing.Any = ...

"""

on initialization of a render job

"""

render_post: typing.Any = ...

"""

on render (after)

"""

render_pre: typing.Any = ...

"""

on render (before)

"""

render_stats: typing.Any = ...

"""

on printing render statistics

"""

render_write: typing.Any = ...

"""

on writing a render frame (directly after the frame is written)

"""

save_post: typing.Any = ...

"""

on saving a blend file (after)

"""

save_pre: typing.Any = ...

"""

on saving a blend file (before)

"""

undo_post: typing.Any = ...

"""

on loading an undo step (after)

"""

undo_pre: typing.Any = ...

"""

on loading an undo step (before)

"""

version_update: typing.Any = ...

"""

on ending the versioning code

"""

persistent: typing.Any = ...

"""

Function decorator for callback functions not to be removed when loading new files

"""



"""


Geometry Utilities (mathutils.geometry)
***************************************

The Blender geometry module

:func:`area_tri`

:func:`barycentric_transform`

:func:`box_fit_2d`

:func:`box_pack_2d`

:func:`convex_hull_2d`

:func:`distance_point_to_plane`

:func:`interpolate_bezier`

:func:`intersect_line_line`

:func:`intersect_line_line_2d`

:func:`intersect_line_plane`

:func:`intersect_line_sphere`

:func:`intersect_line_sphere_2d`

:func:`intersect_plane_plane`

:func:`intersect_point_line`

:func:`intersect_point_quad_2d`

:func:`intersect_point_tri`

:func:`intersect_point_tri_2d`

:func:`intersect_ray_tri`

:func:`intersect_sphere_sphere_2d`

:func:`normal`

:func:`points_in_planes`

:func:`tessellate_polygon`

:func:`volume_tetrahedron`

"""

import typing

import mathutils

def area_tri(v1: mathutils.Vector, v2: mathutils.Vector, v3: mathutils.Vector) -> float:

  """

  Returns the area size of the 2D or 3D triangle defined.

  """

  ...

def barycentric_transform(point: mathutils.Vector, tri_a1: mathutils.Vector, tri_a2: mathutils.Vector, tri_a3: mathutils.Vector, tri_b1: mathutils.Vector, tri_b2: mathutils.Vector, tri_b3: mathutils.Vector) -> typing.Any:

  """

  Return a transformed point, the transformation is defined by 2 triangles.

  """

  ...

def box_fit_2d(points: typing.List[typing.Any]) -> float:

  """

  Returns an angle that best fits the points to an axis aligned rectangle

  """

  ...

def box_pack_2d(boxes: typing.List[typing.Any]) -> typing.Tuple[typing.Any, ...]:

  """

  Returns the normal of the 3D tri or quad.

  """

  ...

def convex_hull_2d(points: typing.List[typing.Any]) -> typing.List[int]:

  """

  Returns a list of indices into the list given

  """

  ...

def distance_point_to_plane(pt: mathutils.Vector, plane_co: mathutils.Vector, plane_no: mathutils.Vector) -> float:

  """

  Returns the signed distance between a point and a plane    (negative when below the normal).

  """

  ...

def interpolate_bezier(knot1: mathutils.Vector, handle1: mathutils.Vector, handle2: mathutils.Vector, knot2: mathutils.Vector, resolution: int) -> typing.List[mathutils.Vector]:

  """

  Interpolate a bezier spline segment.

  """

  ...

def intersect_line_line(v1: mathutils.Vector, v2: mathutils.Vector, v3: mathutils.Vector, v4: mathutils.Vector) -> typing.Tuple[typing.Any, ...]:

  """

  Returns a tuple with the points on each line respectively closest to the other.

  """

  ...

def intersect_line_line_2d(lineA_p1: mathutils.Vector, lineA_p2: mathutils.Vector, lineB_p1: mathutils.Vector, lineB_p2: mathutils.Vector) -> mathutils.Vector:

  """

  Takes 2 segments (defined by 4 vectors) and returns a vector for their point of intersection or None.

  Warning: Despite its name, this function works on segments, and not on lines.

  """

  ...

def intersect_line_plane(line_a: mathutils.Vector, line_b: mathutils.Vector, plane_co: mathutils.Vector, plane_no: mathutils.Vector, no_flip: typing.Any = False) -> mathutils.Vector:

  """

  Calculate the intersection between a line (as 2 vectors) and a plane.
Returns a vector for the intersection or None.

  """

  ...

def intersect_line_sphere(line_a: mathutils.Vector, line_b: mathutils.Vector, sphere_co: mathutils.Vector, sphere_radius: typing.Any, clip: typing.Any = True) -> typing.Any:

  """

  Takes a line (as 2 points) and a sphere (as a point and a radius) and
returns the intersection

  """

  ...

def intersect_line_sphere_2d(line_a: mathutils.Vector, line_b: mathutils.Vector, sphere_co: mathutils.Vector, sphere_radius: typing.Any, clip: typing.Any = True) -> typing.Any:

  """

  Takes a line (as 2 points) and a sphere (as a point and a radius) and
returns the intersection

  """

  ...

def intersect_plane_plane(plane_a_co: mathutils.Vector, plane_a_no: mathutils.Vector, plane_b_co: mathutils.Vector, plane_b_no: mathutils.Vector) -> typing.Tuple[typing.Any, ...]:

  """

  Return the intersection between two planes

  """

  ...

def intersect_point_line(pt: mathutils.Vector, line_p1: mathutils.Vector, line_p2: typing.Any) -> typing.Any:

  """

  Takes a point and a line and returns a tuple with the closest point on the line and its distance from the first point of the line as a percentage of the length of the line.

  """

  ...

def intersect_point_quad_2d(pt: mathutils.Vector, quad_p1: mathutils.Vector, quad_p2: mathutils.Vector, quad_p3: mathutils.Vector, quad_p4: mathutils.Vector) -> int:

  """

  Takes 5 vectors (using only the x and y coordinates): one is the point and the next 4 define the quad,
only the x and y are used from the vectors. Returns 1 if the point is within the quad, otherwise 0.
Works only with convex quads without singular edges.

  """

  ...

def intersect_point_tri(pt: mathutils.Vector, tri_p1: mathutils.Vector, tri_p2: mathutils.Vector, tri_p3: mathutils.Vector) -> mathutils.Vector:

  """

  Takes 4 vectors: one is the point and the next 3 define the triangle.

  """

  ...

def intersect_point_tri_2d(pt: mathutils.Vector, tri_p1: mathutils.Vector, tri_p2: mathutils.Vector, tri_p3: mathutils.Vector) -> int:

  """

  Takes 4 vectors (using only the x and y coordinates): one is the point and the next 3 define the triangle. Returns 1 if the point is within the triangle, otherwise 0.

  """

  ...

def intersect_ray_tri(v1: mathutils.Vector, v2: mathutils.Vector, v3: mathutils.Vector, ray: mathutils.Vector, orig: mathutils.Vector, clip: bool = True) -> mathutils.Vector:

  """

  Returns the intersection between a ray and a triangle, if possible, returns None otherwise.

  """

  ...

def intersect_sphere_sphere_2d(p_a: mathutils.Vector, radius_a: float, p_b: mathutils.Vector, radius_b: float) -> typing.Tuple[typing.Any, ...]:

  """

  Returns 2 points on between intersecting circles.

  """

  ...

def normal(vectors: typing.Sequence[typing.Any]) -> mathutils.Vector:

  """

  Returns the normal of a 3D polygon.

  """

  ...

def points_in_planes(planes: typing.List[mathutils.Vector]) -> typing.Any:

  """

  Returns a list of points inside all planes given and a list of index values for the planes used.

  """

  ...

def tessellate_polygon(veclist_list: typing.Any) -> typing.List[typing.Any]:

  """

  Takes a list of polylines (each point a vector) and returns the point indices for a polyline filled with triangles.

  """

  ...

def volume_tetrahedron(v1: mathutils.Vector, v2: mathutils.Vector, v3: mathutils.Vector, v4: mathutils.Vector) -> float:

  """

  Return the volume formed by a tetrahedron (points can be in any order).

  """

  ...



"""


Fluid Operators
***************

:func:`bake`

:func:`preset_add`

"""

import typing

def bake() -> None:

  """

  Bake fluid simulation

  """

  ...

def preset_add(name: str = '', remove_name: bool = False, remove_active: bool = False) -> None:

  """

  Add or remove a Fluid Preset

  """

  ...



"""


Ui Operators
************

:func:`assign_default_button`

:func:`button_execute`

:func:`copy_data_path_button`

:func:`copy_python_command_button`

:func:`copy_to_selected_button`

:func:`drop_color`

:func:`editsource`

:func:`edittranslation_init`

:func:`eyedropper_color`

:func:`eyedropper_colorramp`

:func:`eyedropper_colorramp_point`

:func:`eyedropper_depth`

:func:`eyedropper_driver`

:func:`eyedropper_id`

:func:`jump_to_target_button`

:func:`override_remove_button`

:func:`override_type_set_button`

:func:`reloadtranslation`

:func:`reports_to_textblock`

:func:`reset_default_button`

:func:`unset_property_button`

"""

import typing

def assign_default_button() -> None:

  """

  Set this property's current value as the new default

  """

  ...

def button_execute(skip_depressed: bool = False) -> None:

  """

  Presses active button

  """

  ...

def copy_data_path_button(full_path: bool = False) -> None:

  """

  Copy the RNA data path for this property to the clipboard

  """

  ...

def copy_python_command_button() -> None:

  """

  Copy the Python command matching this button

  """

  ...

def copy_to_selected_button(all: bool = True) -> None:

  """

  Copy property from this object to selected objects or bones

  """

  ...

def drop_color(color: typing.Tuple[float, float, float] = (0.0, 0.0, 0.0), gamma: bool = False) -> None:

  """

  Drop colors to buttons

  """

  ...

def editsource() -> None:

  """

  Edit UI source code of the active button

  """

  ...

def edittranslation_init() -> None:

  """

  Edit i18n in current language for the active button

  """

  ...

def eyedropper_color(use_accumulate: bool = True) -> None:

  """

  Sample a color from the Blender Window to store in a property

  """

  ...

def eyedropper_colorramp() -> None:

  """

  Sample a color band

  """

  ...

def eyedropper_colorramp_point() -> None:

  """

  Point-sample a color band

  """

  ...

def eyedropper_depth() -> None:

  """

  Sample depth from the 3D view

  """

  ...

def eyedropper_driver(mapping_type: str = 'SINGLE_MANY') -> None:

  """

  Pick a property to use as a driver target

  """

  ...

def eyedropper_id() -> None:

  """

  Sample a data-block from the 3D View to store in a property

  """

  ...

def jump_to_target_button() -> None:

  """

  Switch to the target object or bone

  """

  ...

def override_remove_button(all: bool = True) -> None:

  """

  Remove an override operation

  """

  ...

def override_type_set_button(all: bool = True, type: str = 'REPLACE') -> None:

  """

  Create an override operation, or set the type of an existing one

  """

  ...

def reloadtranslation() -> None:

  """

  Force a full reload of UI translation

  """

  ...

def reports_to_textblock() -> None:

  """

  Write the reports

  """

  ...

def reset_default_button(all: bool = True) -> None:

  """

  Reset this property's value to its default value

  """

  ...

def unset_property_button() -> None:

  """

  Clear the property and use default or generated value in operators

  """

  ...



"""


GPU Select (gpu.select)
***********************

This module provides access to selection.

load_id(id)

  Set the selection ID.

  :param id:        
    Number (32-bit unsigned int).

  :type select:     
    int

"""

import typing
